{*******************************************************************}
{*                                                                 *}
{*  UNIT:  C R T            V4.01           03.07.98, 12.01.99     *}
{*                                                                 *}
{*  compiler: Delphi-4                    testet with Win-NT 4.0   *}
{*                                                                 *}
{*-----------------------------------------------------------------*}
{*                                                                 *}
{*  Implementation of Turbo/Borland Pascal compatible CRT-Unit     *}
{*  for Delphi-3 using Win32 calls (WAPI).                         *}
{*  Some extended functions are added.                             *}
{*                                                                 *}
{*  procedures and functions:                                      *}
{*  -------------------------------------------------------------- *}
{*  AssignCRT        assign crt output to file                     *}
{*  TextMode         set / reset text mode                         *}
{*  ClrEol           clear end of line                             *}
{*  ClrScr           clear screen window and reset cursor          *}
{*  GotoXY           go to text position X,Y (relative)            *}
{*  InsLine          insert line at current cursor position        *}
{*  DelLine          delete line at current cursor position        *}
{*  WhereX           get X-Position of text cursor (relative)      *}
{*  WhereY           get Y-Position of text cursor (relative)      *}
{*  Window           set text window                               *}
{*                                                                 *}
{*  TextBackground   set text background color (0..7)              *}
{*  TextColor        set text foreground color (0..15)             *}
{*  HighVideo        set bright text color                         *}
{*  LowVideo         set dark text color                           *}
{*  NormVideo        set start text color and background           *}
{*                                                                 *}
{*  Delay            wait time in milliseconds                     *}
{*  KeyPressed       check if key was pressed                      *}
{*  ReadKey          read last key char                            *}
{*                                                                 *}
{*  Sound            play sound with given Hertz                   *}
{*  NoSound          stop playing sound                            *}
{*                                                                 *}
{*                                                                 *}
{*  additional procedures and functions:                           *}
{*  -------------------------------------------------------------- *}
{*  Sound2              play sound with given Hertz and duration   *}
{*  FlushInputBuffer    flush keyboard and all other events        *}
{*  GetInputMode        see WIN32.HLP GetConsoleMode               *}
{*  GetOutputMode       see WIN32.HLP GetConsoleMode               *}
{*  SetCodePage         see WIN32.HLP SetConsoleCP                 *}
{*  SetInputMode        see WIN32.HLP SetConsoleMode               *}
{*  SetOutputMode       see WIN32.HLP SetConsoleMode               *}
{*  HideCursor          set text cursor invisible                  *}
{*  ShowCursor          set text cursor visible                    *}
{*  SetInsertMode       set insert mode and change cursor size     *}
{*  EndCRT              ask and wait for key input when not piping *}
{*                                                                 *}
{*  additional global variables:                                   *}
{*  -------------------------------------------------------------- *}
{*  screenSize: TPoint       actual screen size (columns,lines)    *}
{*  soundDuration: integer   sound duration in milliseconds        *}
{*  inputPiping: boolean     true if input piping is active        *}
{*  outputPiping: boolean    true if output piping is active       *}
{*  insertMode: boolean      true if insert mode is active         *}
{*  german: boolean          true if german language is used       *}
{*                                                                 *}
{*******************************************************************}
{$I-,S-}

{$D+}
{$ifopt D+}
  {$define xTest1       main function test}
  {$define xTest2       sub function test }
  {$define xTest3       key input test}
{$endif}

UNIT CRT;

INTERFACE  uses  Windows, SysUtils;

{$ifdef win32}

const
  { CRT modes }
  BW40          = 0;            { 40x25 B/W on Color Adapter }
  CO40          = 1;            { 40x25 Color on Color Adapter }
  BW80          = 2;            { 80x25 B/W on Color Adapter }
  CO80          = 3;            { 80x25 Color on Color Adapter }
  Mono          = 7;            { 80x25 on Monochrome Adapter }
  Font8x8       = 256;          { Add-in for ROM font }

  { Foreground and background color constants }
  Black         = 0;
  Blue          = 1;
  Green         = 2;
  Cyan          = 3;
  Red           = 4;
  Magenta       = 5;
  Brown         = 6;
  LightGray     = 7;

  { Foreground color constants }
  DarkGray      = 8;
  LightBlue     = 9;
  LightGreen    = 10;
  LightCyan     = 11;
  LightRed      = 12;
  LightMagenta  = 13;
  Yellow        = 14;
  White         = 15;

  { Add-in for blinking }
  Blink         = 128;

var
  { Interface variables }
  checkBreak: boolean  = true;    // Enable Ctrl-Break
  checkEOF: boolean    = false;   // Enable Ctrl-Z
  checkSnow: boolean   = false;   // Enable snow filtering
  directVideo: boolean = true;    // Enable direct video addressing
  lastMode: word  = 3;            // Current text mode
  textAttr: byte  = white;        // Current text attribute
  windMin: word   = 0;            // Window upper left coordinates
  windMax: word   = 0;            // Window lower right coordinates

procedure AssignCrt(var F: Text);
procedure ClrEol;
procedure ClrScr;
procedure InsLine;
procedure DelLine;
procedure GotoXY(const x,y:integer);
procedure HighVideo;
procedure LowVideo;
procedure NormVideo;
procedure TextBackground(const Color: word);
procedure TextColor(const Color: word);
procedure TextMode(mode: integer);
function WhereX: integer;
function WhereY: integer;
procedure Window(x1,y1, x2,y2: byte);

procedure Delay(const ms: integer);
function KeyPressed: boolean;
function ReadKey: Char;

procedure Sound(Hz:word);
procedure NoSound;

{-- additional functions and variables --}
procedure Sound2 (Hz:word; duration: integer);
procedure HideCursor;
procedure ShowCursor;
procedure SetInsertMode (insert: boolean);
procedure FlushInputBuffer;
procedure GetInputMode (var mode: DWORD);
procedure GetOutputMode (var mode: DWORD);
procedure SetCodePage (codePage: word);
procedure SetInputMode (mode: DWORD);
procedure SetOutputMode (mode: DWORD);
procedure EndCRT;

const BEL=#7;    FF=#12;     CRLF=#13#10;
       BS=#8;    CR=#13;
       LF=#10;  ESC=#27;
var
  screenSize:   tPoint = (x:80; y:25);  // actual screen size
  soundDuration:integer = 100;    // sound duration in milliseconds
  inputPiping:  boolean = false;  // true if input piping is active
  outputPiping: boolean = false;  // true if output piping is active
  insertMode:   boolean = true;   // true if insert mode is active
  german:       boolean = false;  // true if german language is used
  inputMode:    DWORD   = 0;      // see WAPI-Help SetConsoleMode
  outputMode:   DWORD   = 0;

{$endif win32}

{*******************************************************************}
IMPLEMENTATION

var
  hInput:  tHandle = 0;           // input handle
  hOutput: tHandle = 0;           // output handle
  hError:  tHandle = 0;           // error handle
  crtInfo: TConsoleScreenBufferInfo;
  cursorInfo: tConsoleCursorInfo; // cursor height % / visible
  normAttr: word;                 // starting string attributes
  oldCP: integer;                 // starting code page
  cPos: tCoord = (x:0; y:0);      // cursor position
  clip: tSmallRect = (left: 0;    // clipping window
                      top: 0;
                      right:79;
                      bottom:24);
  fillInfo: tCharInfo             // character fill attributes
    = (asciiChar: ' ';
       attributes: 0);

{*******************************************************************}
{ set absolute cursor position}
{  x: 0 .. screenSize.x-1,   y: 0 .. screenSize.y-1}
{-------------------------------------------------------------------}
procedure SetCursor(const x,y:integer);
begin
  cPos.x := x;
  cPos.y := y;
  SetConsoleCursorPosition(hOutput, cPos);
end;
{*******************************************************************}
{ delete text line of active window at given line position}
{-------------------------------------------------------------------}
procedure DeleteLine (y: integer);
var  newPos: tCoord;
     sourceRect: tSmallRect;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  newPos.x := clip.left;
  newPos.y := y;
  sourceRect := clip;
  sourceRect.top := y+1;
  fillInfo.attributes := crtInfo.wAttributes;
  ScrollConsoleScreenBuffer(hOutput,sourceRect,nil,newPos,fillInfo);
end;
{*******************************************************************}
procedure LineFeed;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  with crtInfo
  do begin
    cPos := dwCursorPosition;
    if cPos.y < clip.bottom
    then SetCursor (clip.left, cPos.y+1)
    else begin
      DeleteLine (clip.top);
      SetCursor (clip.left, cPos.y);
    end;
  end;
end;
{*******************************************************************}
procedure CRT_IO(var t: tTextRec);
var   byteNo, bno: DWORD;   pp: pchar;   ci: cardinal;
      ch: char;
begin
  with t do case mode of
    fmInOut, fmOutput: begin
      if bufPos = 0 then exit;   { ===> }
      bufPtr[bufPos] := #0;
      bno := 0;
      pp := bufptr;
      for ci := 0 to bufPos
      do begin
        ch := bufPtr[ci];
        if ch >= #32    {>= blanc ?}
        then inc (bno)
        else begin
          if bno > 0
          then WriteConsole(handle,pp,bno,byteNo, nil);
          pp := @bufPtr[ci+1];
          bno := 0;
          case ch of
            CR:  SetCursor (clip.left,WhereY-1);
            LF:  LineFeed;
            FF:  ClrScr;
            BS:  SetCursor (WhereX-1,WhereY);
            BEL: Sound(440);
          end;
        end;
      end;
      BufPos := 0;
    end;

    fmInput: BufEnd := cardinal(ReadConsole(handle,BufPtr,BufSize,byteNo, nil));
  end;
end;

{*******************************************************************}
procedure AssignCrt(var f: Text);
begin
{$ifdef Test1}
  writeln ('CRT.AssignCrt: ');
{$endif}
  tTextRec(f).InOutFunc:=@CRT_IO;
  tTextRec(f).FlushFunc:=@CRT_IO;
end;
{*******************************************************************}
procedure TextMode(mode: integer);
begin
  lastMode := mode;
end;
{*******************************************************************}
procedure ClrEol;
var   len,cw: DWORD;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  with crtInfo
  do begin
    cPos := dwCursorPosition;
    len := clip.right - cPos.x + 1;
    if len > 0
    then begin
      FillConsoleOutputAttribute(hOutput,textAttr,len,cPos,cw);
      FillConsoleOutputCharacter(hOutput,' '     ,len,cPos,cw);
    end;
  end;
end;
{*******************************************************************}
procedure ClrScr;
var   yi, cw, cl: DWORD;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  cl := clip.right - clip.left + 1;
  SetCursor (clip.left, clip.top);
  for yi := clip.top to clip.bottom
  do begin
    cPos.y := yi;
    FillConsoleOutputAttribute(hOutput, textAttr, cl, cPos, cw);
    FillConsoleOutputCharacter(hOutput, ' '     , cl, cPos, cw);
  end;
end;
{*******************************************************************}
Function WhereX: integer;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  result := tCoord(crtInfo.dwCursorPosition).x+1
end;
{*******************************************************************}
Function WhereY: integer;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  result := tCoord(crtInfo.dwCursorPosition).y+1
end;
{*******************************************************************}
{ define text window limits and set cursor to left top position}
{  x: 1 .. screenSize.x, y: 1 .. screenSize.y}
{-------------------------------------------------------------------}
procedure Window(x1,y1, x2,y2: byte);
begin
  with crtInfo
  do begin
    if x1 < 1 then clip.left := 0 else clip.left := x1 - 1;
    if y1 < 1 then clip.top  := 0 else clip.top  := y1 - 1;
    if x2 > dwSize.x
    then clip.right := dwSize.x - 1
    else clip.right := x2 - 1;
    if y2 > dwSize.y
    then clip.bottom := dwSize.y - 1
    else clip.bottom := y2 - 1;
    cPos.x := clip.left;
    cPos.y := clip.top;
    {only for compatipility}
    SetCursor (cPos.x,cPos.y);
    WindMin := byte(clip.top)   + byte(clip.left)   shl 8;
    WindMax := byte(clip.right) + byte(clip.bottom) shl 8;
  end;
end;
{*******************************************************************}
{ set cursor position relative to active window}
{  x: 1 .. screenSize.x,   y: 1 .. screenSize.y}
{-------------------------------------------------------------------}
procedure GotoXY(const x,y:integer);
begin
  cPos.x := clip.left + x - 1;    {window grenzen checken ???}
  cPos.y := clip.top  + y - 1;
  SetConsoleCursorPosition(hOutput, cPos);
end;
{*******************************************************************}
{ insert line at actual cursor postion}
{-------------------------------------------------------------------}
procedure InsLine;
var   sourceRect: tSmallRect;
      newPos: tCoord;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  with crtInfo
  do begin
    newPos.x := clip.left;
    newPos.y := dwCursorPosition.y+1;
    sourceRect := clip;
    sourceRect.top := dwCursorPosition.y;
    dec (sourceRect.bottom);
    fillInfo.attributes := wAttributes;
    ScrollConsoleScreenBuffer(hOutput,sourceRect,nil,newPos,fillInfo);
  end;
end;
{*******************************************************************}
{ delete line at actual cursor postion}
{-------------------------------------------------------------------}
procedure DelLine;
begin
  GetConsoleScreenBufferInfo(hOutput,crtInfo);
  DeleteLine (crtInfo.dwCursorPosition.y);
end;
{*******************************************************************}
procedure TextBackground(const Color:word);
begin
  textAttr := (color shl 4) or (textAttr and $f);
  SetConsoleTextAttribute(hOutput,textAttr);
end;
{*******************************************************************}
procedure TextColor(const Color:word);
begin
  textAttr := (color and $f) or (textAttr and $f0);
  SetConsoleTextAttribute(hOutput,textAttr);
end;
{*******************************************************************}
procedure HighVideo;
begin
  textAttr := textAttr or $8;
  SetConsoleTextAttribute(hOutput,textAttr);
end;
{*******************************************************************}
procedure LowVideo;
begin
  textAttr := textAttr and $f7;
  SetConsoleTextAttribute(hOutput,textAttr);
end;
{*******************************************************************}
procedure NormVideo;
begin
  textAttr := normAttr;
  SetConsoleTextAttribute(hOutput,textAttr);
end;
{*******************************************************************}
procedure Delay (const ms:integer);
begin
  sleep(ms);
end;
{*******************************************************************}
var   lastChar: char = #0;
      specialKey: boolean = false;

function KeyPressed: boolean;
var   numberOfEvents, ei, NumRead:  DWORD;
      inputRec: tInputRecord;
begin
  if lastChar <> #0
  then KeyPressed := TRUE
  else begin
    KeyPressed := FALSE;
    GetNumberOfConsoleInputEvents (hInput, numberOfEvents);
    if numberOfEvents > 0
    then for ei := 1 to numberOfEvents
    do with inputRec
    do begin
      ReadConsoleInput(hInput, inputRec, 1, NumRead);
      if inputRec.EventType = KEY_EVENT
      then with inputRec.event.KeyEvent
      do begin
{$ifdef Test3}
        if event.KeyEvent.bKeyDown
        then writeln ('ev=',numberofevents
          ,' keystate=', IntToHex(dwControlKeyState,4), dwControlKeyState:8
          ,' A=', ord(asciiChar):3, ' "',asciiChar:1,'"'
          ,' k=', wVirtualKeyCode:3
          ,' s=', wVirtualScanCode:3
          ,event.KeyEvent.bKeyDown:6);
{$endif}
        if event.KeyEvent.bKeyDown
        then begin
          if ((dwControlKeyState and $1F) = 0)
          or (AsciiChar <> #0)
          then begin
            RESULT := true;
            lastChar := AsciiChar;
            specialKey := lastChar = #0;
            if specialKey
            then lastChar := Chr(wVirtualScanCode);
          end;
          break;
        end;
      end;
    end;
  end;
end;
{*******************************************************************}
function ReadKey: char;
begin
  if lastChar <> #0
  then begin
    result := lastChar;
    lastChar := #0;
    specialKey := FALSE;
  end
  else begin
    repeat until KeyPressed;
    if specialKey
    then result := #0
    else begin
      result := lastChar;
      lastChar := #0;
    end;
  end;
{
  write (result,ord(result):4,'   ');
  sound (888);
}
end;
{*******************************************************************}
procedure Sound (Hz:word);
begin
  windows.beep (Hz, soundDuration);
end;
{*******************************************************************}
procedure NoSound;
begin
  windows.beep (0,0);
end;

{=====================================}
{ additional procedures and functions }
{=====================================}

{*******************************************************************}
procedure Sound2 (Hz:word; duration: integer);
begin
  windows.beep (Hz, duration)
end;
{*******************************************************************}
procedure FlushInputBuffer;
begin
  FlushConsoleInputBuffer (hInput);
end;
{-------------------------------------------------------------------}
procedure GetInputMode (var mode: DWORD);
begin
  GetConsoleMode (hInput, mode);
end;
{-------------------------------------------------------------------}
procedure GetOutputMode (var mode: DWORD);
begin
  GetConsoleMode (hOutput, mode);
end;
{-------------------------------------------------------------------}
procedure SetCodePage (codePage: word);
begin
  SetConsoleCP (codePage);
end;
{-------------------------------------------------------------------}
procedure SetInputMode (mode: DWORD);
begin
  SetConsoleMode (hInput, mode);
  GetConsoleMode (hInput, inputMode);
end;
{-------------------------------------------------------------------}
procedure SetOutputMode (mode: DWORD);
begin
  SetConsoleMode (hOutput, mode);
  GetConsoleMode (hOutput, outputMode);
end;
{-------------------------------------------------------------------}
procedure HideCursor;
begin
  cursorInfo.bVisible := false;
  SetConsoleCursorInfo (hOutput, cursorInfo);
end;
{-------------------------------------------------------------------}
procedure ShowCursor;
begin
  cursorInfo.bVisible := true;
  SetConsoleCursorInfo (hOutput, cursorInfo);
end;
{-------------------------------------------------------------------}
procedure SetInsertMode (insert: boolean);
begin
  InsertMode := insert;
  if insert
  then cursorInfo.dwSize := 25
  else cursorInfo.dwSize := 100;
  SetConsoleCursorInfo (hOutput, cursorInfo);
end;
{-------------------------------------------------------------------}
procedure EndCRT;
begin
  if isConsole and not outputPiping
  then begin
    SetFocus(GetCurrentProcess);
    NormVideo;
    if german
    then write(#13#10'Bitte eine Taste drcken... ')
    else write(#13#10'Press any key... ');
    FlushInputBuffer;
    ReadKey;
    FlushInputBuffer;
  end;
end;
{-------------------------------------------------------------------}
procedure InitCRT;
begin
  SetActiveWindow(0);
  hInput  := GetStdHandle(STD_INPUT_HANDLE);
  hOutput := GetStdHandle(STD_OUTPUT_HANDLE);
  hError  := GetStdHandle(STD_ERROR_HANDLE);
  inputPiping  := GetFileType(hInput) = FILE_TYPE_PIPE;
  outputPiping := not GetConsoleScreenBufferInfo(hOutput,crtInfo);
  if not outputPiping
  then with crtInfo
  do begin
    textAttr := wAttributes;
    normAttr := wAttributes;
    clip := srWindow;
    WindMax := byte(clip.right) + byte(clip.bottom) shl 8;
  end;
  GetConsoleCursorInfo (hOutput, cursorInfo);
  screenSize.x := crtInfo.dwSize.x;
  screenSize.y := crtInfo.dwSize.y;
  oldCP := GetConsoleCP;         // get code page
  german := (LoWord(GetUserDefaultLangID) and $3ff) = $07;
  GetConsoleMode (hInput, inputMode);
  GetConsoleMode (hOutput, outputMode);
  AssignCrt (OUTPUT);
end;
{*******************************************************************}
initialization
  InitCRT;
{$ifdef Test1}
  with crtInfo
  do writeln ('CRT.initialization: ',dwSize.x,'*',dwSize.y);
{$endif}
{*******************************************************************}
finalization
  NoSound;
{$ifdef Test1}
  writeln ('CRT.finalization');
  delay (200);
{$endif}
  SetConsoleOutputCP(oldCP);
end.

