{*******************************************************************}
{*                                                                 *}
{*  UNIT:  D O S            V4.01           03.07.98, 12.01.99     *}
{*                                                                 *}
{*  compiler: Delphi-4                    testet with Win-NT 4.0   *}
{*                                                                 *}
{*-----------------------------------------------------------------*}
{*                                                                 *}
{*  Implementation of Turbo/Borland Pascal 7.0 compatible DOS-Unit *}
{*  for Delphi-4 using Win32 calls (WAPI).                         *}
{*  Some extended functions are added.                             *}
{*                                                                 *}
{*  procedures and functions:                                      *}
{*  -------------------------------------------------------------- *}
{*  DosVersion       return DOS version number                     *}
{*  Intr             executes a specified software interrupt       *}
{*  MsDos            invokes DOS function call handler             *}
{*  GetDate          return current date of operating system       *}
{*  GetTime          return current time of operating system       *}
{*  GetVerify        return state of the verify flag in DOS        *}
{*  SetVerify        sets the state of the verify flag in DOS      *}
{*  DiskFree         return free bytes of given drive              *}
{*  DiskSize         return disk size in bytes                     *}
{*  UnpackTime       convert packed date/time into DateTime        *}
{*  PackTime         convert DateTime into packed date/time        *}
{*  FSearch          search for file given by directory pathes     *}
{*  FExpand          expand filename into full qualified file name *}
{*  FSplit           split file name into dir / name / ext         *}
{*  GetEnv           get dos environment variable string           *}
{*  SwapVectors      (nothing to implement)                        *}
{*  Exec             execute external program                      *}
{*  DosExitCode      return dos exit code set by Exec/Keep         *}
{*  GetFAttr         get file attributes                           *}
{*  SetFAttr         set file attribute                            *}
{*  GetFTime         get file time                                 *}
{*  SetFTime         set file time                                 *}
{*  FindFirst        return first matching file                    *}
{*  FindNext         return next matching file                     *}
{*                                                                 *}
{*  additional procedures and functions:                           *}
{*  -------------------------------------------------------------- *}
{*  SetEnv           set dos environment variable string           *}
{*                                                                 *}
{*  additional global variables:                                   *}
{*  -------------------------------------------------------------- *}
{*  dosError: integer        last dos error                        *}
{*                                                                 *}
{*******************************************************************}
{$I-}

UNIT DOS;

{$ifdef WIN32}

INTERFACE  uses  SysUtils;

const readOnly  = faReadOnly;     {file attributes}
      hidden    = faHidden;
      sysFile   = faSysFile;
      volumeID  = faVolumeID;
      directory = faDirectory;
      archive   = faArchive;
      anyFile   = faAnyFile;

      ExecuteSUCCESS  = $0;
      ExecuteWinError = $3000;

type  { String types }
      ComStr    = string[127];    {Command line string }
      PathStr   = string[79];     {File pathname string }
      DirStr    = string[67];     {Drive and directory string }
      NameStr   = string[8];      {File name string }
      ExtStr    = string[4];      {File extension string }

type  searchRec = tSearchRec;     {Search record used by FindFirst and FindNext}
      fileRec   = tFileRec;       {Typed-file and untyped-file record}
      TextBuf   = array[0..127] of Char;
      TextRec   = tTextRec;

type  DateTime  = record          {used by PackTime and UnpackTime}
        Year, Month, Day,
        Hour, Min, Sec: Word;
      end;

type  Registers = record
        case Integer of
          0: (AX,BX,CX,DX,BP,SI,DI,DS,ES,Flags: Word);
          1: (AL,AH,BL,BH,CL,CH,DL,DH: Byte);
        end;
(*
type  tRegisters32 = record
        case Integer of
          0: (edi, esi, ebp, exx, ebx, edx, ecx, eax: longint;
              flags, es, ds, fs, gs, ip, cs, sp, ss : word);
          1: (di, dih, si, sih, bp, bph, xx, xxh: word;
          case Integer of
            0: (bx, bxh, dx, dxh, cx, cxh, ax, axh: word);
            1: (bl, bh, blh, bhh, dl, dh, dlh, dhh,
               cl, ch, clh, chh, al, ah, alh, ahh: byte));
    end;
*)

const DosError: integer = 0;      {Error status variable}

{ DosVersion returns the DOS version number. The low byte of    }
{ the result is the major version number, and the high byte is  }
{ the minor version number. For example, DOS 3.20 returns 3 in  }
{ the low byte, and 20 in the high byte.                        }
function DosVersion: Word;

{ Intr executes a specified software interrupt with a specified }
{ Registers package.                                            }
procedure Intr (IntNo: Byte; var regs: Registers);

{ MsDos invokes the DOS function call handler with a specified  }
{ Registers package.                                            }
procedure MsDos(var Regs: Registers);

{ GetDate returns the current date set in the operating system. }
{ Ranges of the values returned are: Year 1980-2099, Month      }
{ 1-12, Day 1-31 and DayOfWeek 0-6 (0 corresponds to Sunday).   }
procedure GetDate(var Year,Month,Day,WeekDay: Word);

{ SetDate sets the current date in the operating system. Valid  }
{ parameter ranges are: Year 1980-2099, Month 1-12 and Day      }
{ 1-31. If the date is not valid, the function call is ignored. }
procedure SetDate(Year,Month,Day: Word);

{ GetTime returns the current time set in the operating system. }
{ Ranges of the values returned are: Hour 0-23, Minute 0-59,    }
{ Second 0-59 and Sec100 (hundredths of seconds) 0-99.          }
procedure GetTime(var Hour,Minute,Second,Sec100: Word);

{ SetTime sets the time in the operating system. Valid          }
{ parameter ranges are: Hour 0-23, Minute 0-59, Second 0-59 and }
{ Sec100 (hundredths of seconds) 0-99. If the time is not       }
{ valid, the function call is ignored.                          }
procedure SetTime(Hour,Minute,Second,Sec100: Word);

{ GetCBreak returns the state of Ctrl-Break checking in DOS.    }
{ When off (False), DOS only checks for Ctrl-Break during I/O   }
{ to console, printer, or communication devices. When on        }
{ (True), checks are made at every system call.                 }
procedure GetCBreak(var Break: Boolean);

{ SetCBreak sets the state of Ctrl-Break checking in DOS.       }
procedure SetCBreak(Break: Boolean);

{ GetVerify returns the state of the verify flag in DOS. When   }
{ off (False), disk writes are not verified. When on (True),    }
{ all disk writes are verified to insure proper writing.        }
procedure GetVerify(var Verify: Boolean);

{ SetVerify sets the state of the verify flag in DOS.           }
procedure SetVerify(Verify: Boolean);

{ DiskFree returns the number of free bytes on the specified    }
{ drive number (0=Default,1=A,2=B,..). DiskFree returns -1 if   }
{ the drive number is invalid.                                  }
function DiskFree(Drive: Byte): Longint;

{ DiskSize returns the size in bytes of the specified drive     }
{ number (0=Default,1=A,2=B,..). DiskSize returns -1 if the     }
{ drive number is invalid.                                      }
function DiskSize(Drive: Byte): Longint;

{ UnpackTime converts a 4-byte packed date/time returned by     }
{ FindFirst, FindNext or GetFTime into a DateTime record.       }
procedure UnpackTime(P: Longint; var T: DateTime);

{ PackTime converts a DateTime record into a 4-byte packed      }
{ date/time used by SetFTime.                                   }
procedure PackTime(var T: DateTime; var P: Longint);

{ GetIntVec returns the address stored in the specified         }
{ interrupt vector.                                             }
procedure GetIntVec(IntNo: Byte; var Vector: Pointer);

{ SetIntVec sets the address in the interrupt vector table for  }
{ the specified interrupt.                                      }
procedure SetIntVec(IntNo: Byte; Vector: Pointer);

{ FSearch searches for the file given by Path in the list of    }
{ directories given by DirList. The directory paths in DirList  }
{ must be separated by semicolons. The search always starts     }
{ with the current directory of the current drive. The returned }
{ value is a concatenation of one of the directory paths and    }
{ the file name, or an empty string if the file could not be    }
{ located.                                                      }
function FSearch(Path: PathStr; DirList: String): PathStr;

{ FExpand expands the file name in Path into a fully qualified  }
{ file name. The resulting name consists of a drive letter, a   }
{ colon, a root relative directory path, and a file name.       }
{ Embedded '.' and '..' directory references are removed.       }
function FExpand(Path: PathStr): PathStr;

{ FSplit splits the file name specified by Path into its three  }
{ components. Dir is set to the drive and directory path with   }
{ any leading and trailing backslashes, Name is set to the file }
{ name, and Ext is set to the extension with a preceding dot.   }
{ Each of the component strings may possibly be empty, if Path  }
{ contains no such component.                                   }
procedure FSplit (Path: PathStr; var Dir: DirStr;
                  var Name: NameStr; var Ext: ExtStr);

{ EnvCount returns the number of strings contained in the DOS   }
{ environment.                                                  }
function EnvCount: Integer;

{ EnvStr returns a specified environment string. The returned   }
{ string is of the form "VAR=VALUE". The index of the first     }
{ string is one. If Index is less than one or greater than      }
{ EnvCount, EnvStr returns an empty string.                     }
function EnvStr(Index: Integer): String;

{ GetEnv returns the value of a specified environment variable. }
{ The variable name can be in upper or lower case, but it must  }
{ not include the '=' character. If the specified environment   }
{ variable does not exist, GetEnv returns an empty string.      }
function GetEnv (EnvVar: String): String;

procedure SetEnv (name, value: String);

{ SwapVectors swaps the contents of the SaveIntXX pointers in   }
{ the System unit with the current contents of the interrupt    }
{ vectors. SwapVectors is typically called just before and just }
{ after a call to Exec. This insures that the Exec'd process    }
{ does not use any interrupt handlers installed by the current  }
{ process, and vice versa.                                      }
procedure SwapVectors;

{ Keep (or Terminate Stay Resident) terminates the program and  }
{ makes it stay in memory. The entire program stays in memory,  }
{ including data segment, stack segment, and heap. The ExitCode }
{ corresponds to the one passed to the Halt standard procedure. }
procedure Keep(ExitCode: Word);

{ Exec executes another program. The program is specified by    }
{ the Path parameter, and the command line is specified by the  }
{ CmdLine parameter. To execute a DOS internal command, run     }
{ COMMAND.COM, e.g. "Exec('\COMMAND.COM','/C DIR *.PAS');".     }
{ Note the /C in front of the command. Errors are reported in   }
{ DosError. When compiling a program that uses Exec, be sure    }
{ to specify a maximum heap size as there will otherwise not be }
{ enough memory.                                                }
procedure Exec(Path: PathStr; ComLine: ComStr);

{ DosExitCode returns the exit code of a sub-process. The low   }
{ byte is the code sent by the terminating process. The high    }
{ byte is zero for normal termination, 1 if terminated by       }
{ Ctrl-C, 2 if terminated due to a device error, or 3 if        }
{ terminated by the Keep procedure (function call 31 hex).      }
function DosExitCode: Word;

{ GetFAttr returns the attributes of a file. F must be a file   }
{ variable (typed, untyped or textfile) which has been assigned }
{ a name. The attributes are examined by ANDing with the        }
{ attribute masks defined as constants above. Errors are        }
{ reported in DosError.                                         }
procedure GetFAttr(var F; var Attr: Word);

{ SetFAttr sets the attributes of a file. F must be a file      }
{ variable (typed, untyped or textfile) which has been assigned }
{ a name. The attribute value is formed by adding (or ORing)    }
{ the appropriate attribute masks defined as constants above.   }
{ Errors are reported in DosError.                              }
procedure SetFAttr(var F; Attr: Word);

{ GetFTime returns the date and time a file was last written.   }
{ F must be a file variable (typed, untyped or textfile) which  }
{ has been assigned and opened. The Time parameter may be       }
{ unpacked throgh a call to UnpackTime. Errors are reported in  }
{ DosError.                                                     }
procedure GetFTime(var F; var Time: Longint);

{ SetFTime sets the date and time a file was last written.      }
{ F must be a file variable (typed, untyped or textfile) which  }
{ has been assigned and opened. The Time parameter may be       }
{ created through a call to PackTime. Errors are reported in    }
{ DosError.                                                     }
procedure SetFTime(var F; Time: Longint);

{ FindFirst searches the specified (or current) directory for   }
{ the first entry that matches the specified filename and       }
{ attributes. The result is returned in the specified search    }
{ record. Errors (and no files found) are reported in DosError. }
procedure FindFirst(Path: PathStr; Attr: Word; var F: SearchRec);

{ FindNext returns the next entry that matches the name and     }
{ attributes specified in a previous call to FindFirst. The     }
{ search record must be one passed to FindFirst. Errors (and no }
{ more files) are reported in DosError.                         }
procedure FindNext(var F: SearchRec);

{*******************************************************************}
IMPLEMENTATION  uses  CRT, Windows;

const exitCode: integer = 0;           {Exit Code}

const regsPtr: ^registers = nil;       {pointer to register record}

{*******************************************************************}
procedure NotImplemented (info: string);
begin
  writeln ('sorry, function ', info, ' in Unit DOS not implemented');
  delay (1000);
  RunError (233);   {means function not implemented}
end;

{*******************************************************************}
{ return major version number in low byte and minor version number
{ in high byte of returned value
{-------------------------------------------------------------------}
function DosVersion: Word;
begin
  DosVersion := word(GetVersion);
end;

{$define ProtectedMode}
{$define Windows}
{*******************************************************************}
{ executes software interrupt with a specified Registers package.
{-------------------------------------------------------------------}
procedure Intr(IntNo: Byte; var regs: registers);
begin
{  regsPtr^ := regs;           {copy registers}
  asm
    mov ax, $0300   {DPMI service: emulate real-mode interrupt}
    mov bl, IntNo   {Interrupt to emulate}
    xor bh, bh      {Stop A20 gate reset}
    xor cx, cx      {No parameter passing via PM stack}
 {   les di, regs    {Point es:[di] to register structure}
 {   int $31         {Try to run the interrupt}
(*
    cmc             {Carry flag set on error, return boolean}
    mov ax, 0       {return TRUE if C=0,}
    adc ax, ax      {and FALSE otherwise}
*)
  end;
{  Intr := Flags and FCarry <> 0;}
end;
{*******************************************************************}
{ MS-DOS function call with a specified registers package.
{-------------------------------------------------------------------}
procedure MsDos(var Regs: Registers);
begin
  Intr ($21, regs);
end;
{*******************************************************************}
{ return number of free bytes on given drive (0=Default,1=A,2=B,..).
{-------------------------------------------------------------------}
function DiskFree(Drive: Byte): Longint;
begin
  DiskFree := longint (SysUtils.DiskFree (drive));
end;
{*******************************************************************}
{ return disk size in bytes of given drive (0=Default,1=A,2=B,..).
{-------------------------------------------------------------------}
function DiskSize(Drive: Byte): Longint;
begin
  DiskSize := longint (SysUtils.DiskSize (drive));
end;
{*******************************************************************}
{ return current date of operating system
{-------------------------------------------------------------------}
procedure GetDate (var Year,Month,Day,WeekDay: Word);
var Present: System.TDateTime;
begin
  Present := Now;
  DecodeDate (Present, year, month, day);
  WeekDay := DayOfWeek(Present) - 1;
end;

{*******************************************************************}
{ set current date in the operating system
{ (Year 1980-2099, Month 1-12, Day 1-31)
{-------------------------------------------------------------------}
procedure SetDate(Year,Month,Day: Word);
var   sysTime: TSystemTime;
begin
  GetSystemTime(sysTime);
  sysTime.wYear := year;
  sysTime.wMonth := month;
  sysTime.wDay := day;
  SetSystemTime (sysTime);
end;

{*******************************************************************}
{ return current time of operating system
{-------------------------------------------------------------------}
procedure GetTime (var Hour,Minute,Second,Sec100: Word);
begin
  DecodeTime (Now, hour, minute, second, sec100);
  sec100 := (sec100 + 5) div 10;
end;

{*******************************************************************}
{ set time in the operating system.
{ (Hour 0-23, Minute 0-59, Second 0-59, Sec100 0-99)
{-------------------------------------------------------------------}
procedure SetTime(Hour,Minute,Second,Sec100: Word);
var   sysTime: TSystemTime;
begin
  GetLocalTime(sysTime);
  sysTime.wHour := hour;
  sysTime.wMinute := minute;
  sysTime.wSecond := second;
  sysTime.wMilliseconds := sec100 * 10;
  SetLocalTime (sysTime);
end;

{*******************************************************************}
{ returns state of Ctrl-Break checking in DOS.
{-------------------------------------------------------------------}
procedure GetCBreak (var Break: Boolean);
var   mode: DWORD;
begin
  GetInputMode (mode);
  break := (mode and ENABLE_PROCESSED_INPUT) <> 0;
end;
{*******************************************************************}
{ set state of Ctrl-Break checking in DOS.
{-------------------------------------------------------------------}
procedure SetCBreak(Break: Boolean);
var   mode: DWORD;
begin
  GetInputMode (mode);
  mode := mode and not ENABLE_PROCESSED_INPUT;
  SetInputMode (mode);
end;
{*******************************************************************}
{ return state of the verify flag in DOS.
{-------------------------------------------------------------------}
procedure GetVerify(var Verify: Boolean);
var   regs: registers;
begin
  regs.ax := $5400;
  msdos(regs);
  verify := (regs.ax and $01) <> 0;
end;

{*******************************************************************}
{ SetVerify sets the state of the verify flag in DOS.
{-------------------------------------------------------------------}
procedure SetVerify(Verify: Boolean);
var   regs: registers;
begin
  regs.al := byte(verify);
  regs.ah := $2E;
  msdos(regs);
{  NotImplemented ('SetVerify');}
end;

{*******************************************************************}
{ convert 4-byte packed date/time into a DateTime record }
{-------------------------------------------------------------------}
procedure UnpackTime(P: Longint; var T: DateTime);
var   dt: system.TDateTime;  mSec: word;
begin
  if P = -1
  then FillChar (T, sizeof(T), 0)
  else with T
  do begin
    dt := FileDateToDateTime (P);
    DecodeDate (dt, year, month, day);
    DecodeTime (dt, hour, min, sec, mSec);
  end;
end;

{*******************************************************************}
{ convert a DateTime record into a 4-byte packed date/time }
{-------------------------------------------------------------------}
procedure PackTime(var T: DateTime; var P: Longint);
var   dt: system.TDateTime;  sec100: word;
begin
  sec100 := 0;
  with T
  do dt := EncodeDate (year , month, day) + EncodeTime (hour, min, sec, sec100);
  P := DateTimeToFileDate (dt);
end;

{*******************************************************************}
{ returns expected vector address of interrupt vector table         }
{-------------------------------------------------------------------}
procedure GetIntVec(IntNo: Byte; var Vector: Pointer);
var   regs: registers;
begin
  regs.al := IntNo;
  regs.ah := $25;
  Intr ($21, regs);
  vector := pointer (regs.es shr 16 + regs.di);
end;
{*******************************************************************}
{ sets vector address in the interrupt vector table                 }
{-------------------------------------------------------------------}
procedure SetIntVec(IntNo: Byte; Vector: Pointer);
var   regs: registers;
begin
  regs.al := IntNo;
  regs.ah := $25;
  regs.dx := longint(vector);
  Intr ($21, regs);
end;

{*******************************************************************}
{ searches for the file given by directory pathes }
{-------------------------------------------------------------------}
function FSearch(Path: PathStr; DirList: String): PathStr;
begin
  FSearch := FileSearch (path, dirList);
end;
{*******************************************************************}
{ expands the file name in Path into a fully qualified file name.
{-------------------------------------------------------------------}
function FExpand(Path: PathStr): PathStr;
begin
  FExpand := ExpandFileName (path);
end;
{*******************************************************************}
{ split file name specified by path into
{   Dir    drive:directory\
{   Name   file name without extension
{   Ext    file name extension with a preceding dot
{-------------------------------------------------------------------}
procedure FSplit (Path: PathStr; var Dir: DirStr;
                  var Name: NameStr; var Ext: ExtStr);
begin
  dir  := ExtractFilePath (path);
  name := ExtractFileName (path);
  if pos('.', name) <> 0 then delete (name,pos('.', name),8);
  ext  := ExtractFileExt  (path);
end;

{*******************************************************************}
function EnvCount: Integer;
begin
  NotImplemented ('EnvCount');
  EnvCount := 0;
end;

{*******************************************************************}
function EnvStr(Index: Integer): String;
begin
  NotImplemented ('EnvStr');
end;

{*******************************************************************}
{ get dos environment variable string of actual process }
{-------------------------------------------------------------------}
function GetEnv (EnvVar: String): String;
var env: string[255];
    cp: pChar;
begin
  envVar := envVar + #0;
  cp := @env[1];
  GetEnvironmentVariable (@envVar[1], cp, 255);
  env[0] := char(StrLen (cp));
  result := env;
end;
{*******************************************************************}
{ set dos environment variable for actual process  e.g. TMP=C:\TEMP }
{-------------------------------------------------------------------}
procedure SetEnv (name, value: String);
begin
  name := name + #0;
  value := value + #0;
  SetEnvironmentVariable (@name[1], @value[1]);
  DosError := GetLastError;
end;

{*******************************************************************}
procedure SwapVectors;
begin
  {nothing to implement}
end;

{*******************************************************************}
procedure Keep(ExitCode: Word);
begin
  NotImplemented ('Keep');
end;

{*******************************************************************}
{ Executes an external program (command = Path + ComLine).
{ DosError will be set.
{-------------------------------------------------------------------}
procedure Exec(Path: PathStr; ComLine: ComStr);

var
  execBuffer : Array [0..255] of Char;
  StartupInfo: TStartUpInfo;
  ProcessInfo: TProcessInformation;
  SecurityPAttributes: TSecurityAttributes;
  SecurityTAttributes: TSecurityAttributes;

  procedure ExecError (s: string);
  begin
    if DosError = 0
    then DosError := ExecuteWinError OR GetLastError;
{
    writeln (s:40,':  GetLastError=',GetLastError);
}
  end;

begin
  StrPCopy (execBuffer, Path + ' ' + ComLine + #0);
  SetLastError (0);
  DosError := 0;
  FillChar (StartUpInfo,SizeOf(TStartUpInfo),0);
  FillChar (ProcessInfo,SizeOf(TProcessInformation),0);
  FillChar (SecurityPAttributes, SizeOf (TSecurityAttributes), 0);
  FillChar (SecurityTAttributes, SizeOf (TSecurityAttributes), 0);

  with StartupInfo
  do begin
    cb := SizeOf(StartupInfo);
    dwFlags := STARTF_USESHOWWINDOW or STARTF_FORCEONFEEDBACK;
    wShowWindow := SW_SHOWNORMAL;
  end;

  with SecurityPAttributes do
  begin
    nLength := SizeOf(SecurityPAttributes);
    lpSecurityDescriptor := nil;
    bInheritHandle := TRUE;
  end;
  SecurityTAttributes := SecurityPAttributes;

  if not CreateProcess
    (nil                          // pointer to name of executable module
    ,execBuffer                   // pointer to command line string
    ,@SecurityPAttributes         // pointer to process security attributes
    ,@SecurityTAttributes         // pointer to thread security attributes
    ,TRUE                         // handle inheritance flag
    ,NORMAL_PRIORITY_CLASS        // creation flags
    ,nil                          // pointer to new environment block
    ,nil                          // pointer to current directory name
    ,StartUpInfo                  // pointer to STARTUPINFO
    ,ProcessInfo)                 // pointer to PROCESS_INFORMATION
  then ExecError ('CreateProcess failed');

  case WaitForSingleObject(ProcessInfo.hProcess, INFINITE) of
    WAIT_ABANDONED,
    WAIT_OBJECT_0: ;
    WAIT_TIMEOUT:  ExecError ('WaitForSingleObject time-out');
    WAIT_FAILED:   ExecError ('WaitForSingleObject failed');
  end;

  if DosError = 0
  then GetExitCodeProcess (ProcessInfo.hProcess, DWORD(DosError));

  CloseHandle (ProcessInfo.hProcess);
  CloseHandle (ProcessInfo.hThread);
end;

{*******************************************************************}
{ return dos exit code
{-------------------------------------------------------------------}
function DosExitCode: Word;
begin
  DosExitCode := exitCode;
end;

{*******************************************************************}
{ get attributes of a file / DosError will be set.
{-------------------------------------------------------------------}
procedure GetFAttr(var F; var Attr: Word);      {get file attributes}
begin
  attr := word(FileGetAttr(TTextRec(f).Name));
end;
{*******************************************************************}
{ set attributes of a file / DosError will be set.
{-------------------------------------------------------------------}
procedure SetFAttr(var F; Attr: Word);          {set file attributes}
begin
  DosError := FileSetAttr(TTextRec(f).Name, Attr);
end;

{*******************************************************************}
{ GetFTime returns the date and time a file was last written.
{ DosError will be set.
{-------------------------------------------------------------------}
procedure GetFTime(var F; var Time: Longint);
var   MSec: Word;
      fileDate: system.TDateTime;
      dt: datetime;
begin
  time := FileGetDate(TTextRec(f).Handle);
  if time = -1 then exit;
  fileDate := FileDateToDateTime(time);
  with dt
  do begin
    DecodeDate (fileDate, year, month, day);
    DecodeTime (fileDate, Hour, Min, Sec, MSec);
  end;
  PackTime (dt, time);
end;
{*******************************************************************}
{ SetFTime sets the date and time a file was last written.
{ DosError will be set.
{-------------------------------------------------------------------}
procedure SetFTime(var F; Time: Longint);
begin
  DosError := FileSetDate(TFileRec(f).Handle, Time);
end;

{*******************************************************************}
{ FindFirst returns the first file that matches in the search record.
{ DosError will be set.
{-------------------------------------------------------------------}
procedure FindFirst(Path: PathStr; Attr: Word; var F: SearchRec);
begin
  DosError := SysUtils.FindFirst(Path, Attr, F);
end;
{*******************************************************************}
{ FindNext returns the next file that matches in the search record.
{ DosError will be set.
{-------------------------------------------------------------------}
procedure FindNext(var F: TSearchRec);
begin
  DosError := SysUtils.FindNext(F);
end;

{*******************************************************************}
initialization
{
  GetMem (regsPtr, sizeof (registers));
}
{*******************************************************************}
finalization
{
  FreeMem (regsPtr);
}
end.

{$endif}

